/***************************************************************************


***************************************************************************/
#include <ctype.h>
#include "driver.h"
#include "cpu/m6502/m6502.h"
#include "mess/machine/vc20.h"
#include "6522via.h"


static int via1_portb;
//static int via0_porta, via0_portb, via0ca2, via0cb2, via1_porta, via1ca2, via1cb2;

static UINT8 keyline[8]={0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff};
UINT8 *vc20_memory;
UINT8 *vc20_memory_1000;
UINT8 *vc20_memory_9400;

static void via0_irq (int level)
{
	static int old_level;
	if (level != old_level)
	{
		cpu_set_nmi_line(0, level);
		old_level = level;
	}
}

static int via0_read_ca1(int offset)
{
	return KEY_RESTORE?0:1;
}

static int via0_read_porta(int offset)
{
	int value=0xff;
	if (JOY_BUTTON) value&=~0x20;
	if (JOY_LEFT) value&=~0x10;
	if (JOY_UP) value&=~4;
	if (JOY_DOWN) value&=~8;

	return value;
}

static void via1_irq(int level)
{
	static int old_level;
	if (level != old_level)
	{
		cpu_set_irq_line(0, M6502_INT_IRQ, level);
		old_level = level;
	}
}

static int via1_read_porta(int offset)
{
	int value=0xff;

	if (!(via1_portb&1)) {
		value&=keyline[0];
	}
	if (!(via1_portb&2)) {
		value&=keyline[1];
	}
	if (!(via1_portb&4)) {
		value&=keyline[2];
	}
	if (!(via1_portb&8)) {
		value&=keyline[3];
	}
	if (!(via1_portb&0x10)) {
		value&=keyline[4];
	}
	if (!(via1_portb&0x20)) {
		value&=keyline[5];
	}
	if (!(via1_portb&0x40)) {
		value&=keyline[6];
	}
	if (!(via1_portb&0x80)) {
		value&=keyline[7];
	}
	return value;
}

static int via1_read_portb(int offset)
{
	int value=0xff;

	if (JOY_RIGHT) value&=~0x80;

	return value;
}

static void via1_write_portb(int offset, int data)
{
	via1_portb=data;
}

struct via6522_interface via0= {
	via0_read_porta,
	0,//via0_read_portb,
	via0_read_ca1,
	0,//via0_read_cb1,
	0,//via0_read_ca2,
	0,//via0_read_cb2,
	0,//via0_write_porta,
	0,//via0_write_portb,
	0,//via0_write_ca2,
	0,//via0_write_cb2,
	via0_irq
}, via1= {
	via1_read_porta,
	via1_read_portb,
	0,//via1_read_ca1,
	0,//via1_read_cb1,
	0,//via1_read_ca2,
	0,//via1_read_cb2,
	0,//via1_write_porta,
	via1_write_portb,
	0,//via1_write_ca2,
	0,//via1_write_cb2,
	via1_irq
};

/*************************************
 *
 *		Port handlers.
 *
 *************************************/
void vc20_write(int offset, int data)
{
	if (vc20_memory[offset]==data) return;
	vc20_memory[offset]=data;
	vic6560_addr_w(VC20ADDR2VIC6560ADDR(offset),data);
}

void vc20_write_0400(int offset, int data)
{
	if (vc20_memory[0x400+offset]==data) return;
		vc20_memory[0x400+offset]=data;
		vic6560_addr_w(VC20ADDR2VIC6560ADDR(0x0400+offset),data);
}

void vc20_write_1000(int offset, int data)
{
	if (vc20_memory_1000[offset]==data) return;
	vc20_memory_1000[offset]=data;
	vic6560_addr_w(VC20ADDR2VIC6560ADDR(0x1000+offset),data);
}
void vc20_write_9400(int offset, int data)
{
	if (vc20_memory_9400[offset]!=data) {
		vic6560_addr8_w(offset,data);
		vc20_memory_9400[offset]=data|0xf0;
	}
}

int vic6560_dma_read(int offset)
{
	int value=((int)vc20_memory_9400[offset&0x3ff]&0xf)<<8,
		addr=VIC6560ADDR2VC20ADDR(offset);
	if (addr<0x2000) return vc20_memory[addr]|value;
	if (addr<0x9000) return vc20_memory[addr]|value;
	return value|0xff;
}


static void vc20_common_init_machine(void)
{
	int i;

	// rams look like 0xff when powered on (2114)

	//	vc20_memory[0x288]=0xff;// makes ae's graphics look correctly
	//	vc20_memory[0xd]=0xff; // for moneywars
	for (i=0;i<0x400;i+=0x40) memset(vc20_memory+i,i&0x40?0:0xff,0x40);
//	for (i=0x1000;i<0x2000;i+=0x40) memset(vc20_memory+i,i&0x40?0:0xff,0x40);
	for (i=0x9400;i<0x9800;i+=0x40) memset(vc20_memory+i,i&0x40?0:0xff,0x40);
	for (i=0;i<0x400;i++) vc20_memory_9400[i]|=0xf0;

#if 0
	// i think roms look like 0xff
	for (i=0x400;i<0x1000;i++) vc20_memory[i]=0xff;
	for (i=0;i<0x2000;i++) {
		vc20_memory_2000[i]=0xff;
		vc20_memory_4000[i]=0xff;
		vc20_memory_6000[i]=0xff;
		vc20_memory_a000[i]=0xff;
	}
#endif

	if (RAMIN0X0400) {
		install_mem_write_handler(0,0x400,0xfff,vc20_write_0400);
		install_mem_read_handler(0,0x400,0xfff,MRA_RAM);
	} else {
		install_mem_write_handler(0,0x400,0xfff,MWA_NOP);
		install_mem_read_handler(0,0x400,0xfff,MRA_ROM);

	}
	if (RAMIN0X2000) {
		install_mem_write_handler(0,0x2000,0x3fff,MWA_RAM);
		install_mem_read_handler(0,0x2000,0x3fff,MRA_RAM);
	} else {
		install_mem_write_handler(0,0x2000,0x3fff,MWA_NOP);
		install_mem_read_handler(0,0x2000,0x3fff,MRA_ROM);

	}
	if (RAMIN0X4000) {
		install_mem_write_handler(0,0x4000,0x5fff,MWA_RAM);
		install_mem_read_handler(0,0x4000,0x5fff,MRA_RAM);
	} else {
		install_mem_write_handler(0,0x4000,0x5fff,MWA_NOP);
		install_mem_read_handler(0,0x4000,0x5fff,MRA_ROM);

	}
	if (RAMIN0X6000) {
		install_mem_write_handler(0,0x6000,0x7fff,MWA_RAM);
		install_mem_read_handler(0,0x6000,0x7fff,MRA_RAM);
	} else {
		install_mem_write_handler(0,0x6000,0x7fff,MWA_NOP);
		install_mem_read_handler(0,0x6000,0x7fff,MRA_ROM);

	}
	if (RAMIN0XA000) {
		install_mem_write_handler(0,0xa000,0xbfff,MWA_RAM);
		install_mem_read_handler(0,0xa000,0xbfff,MRA_RAM);
	} else {
		install_mem_write_handler(0,0xa000,0xbfff,MWA_NOP);
		install_mem_read_handler(0,0xa000,0xbfff,MRA_ROM);

	}
	via_config(0,&via0);
	via_config(1,&via1);
	via_0_ca1_w(0,via0_read_ca1(0) );

	vc20_rom_load();
#if 0
	for (i = 0; i < Machine->gamedrv->num_of_floppy_drives; i++)
	{
		/* no floppy name given for that drive ? */
		if (!floppy_name[i]) continue;
		if (!floppy_name[i][0]) continue;
		pc_fdc_file[i] = osd_fopen(Machine->gamedrv->name, floppy_name[i], OSD_FILETYPE_IMAGE, 1);
		/* find the sectors/track and bytes/sector values in the boot sector */
		if (!pc_fdc_file[i] && Machine->gamedrv->clone_of)
			pc_fdc_file[i] = osd_fopen(Machine->gamedrv->clone_of->name, floppy_name[i], OSD_FILETYPE_IMAGE, 1);
		if (pc_fdc_file[i])
		{
		//
		}
	 }
#endif
}

void vic20_init_machine(void)
{
	vic6560_init(vic6560_dma_read);
	vc20_common_init_machine();
}

void vc20_init_machine(void)
{
	vic6561_init(vic6560_dma_read);
	vc20_common_init_machine();
}

void vc20_shutdown_machine(void)
{
#if 0
	int i;

	for (i = 0; i < Machine->gamedrv->num_of_floppy_drives; i++)
	{
		if (pc_fdc_file[i]) osd_fclose(pc_fdc_file[i]);
		pc_fdc_file[i] = 0;
	 }
#endif
}

int vc20_rom_load(void)
{
	 int result	= 0;
	 FILE *fp;
	 int size, i, read;
	 char *cp;
	 int addr=0;

	 for (i=0;(i<=2)&&(rom_name[i]!=NULL)&&(strlen(rom_name[i])!=0); i++) {

		 if(!vc20_rom_id(rom_name[i],rom_name[i])) continue;
		 fp = osd_fopen(Machine->gamedrv->name, rom_name[i], OSD_FILETYPE_IMAGE_R, 0);
		 if(!fp) {
			if (errorlog) fprintf(errorlog,"%s file not found\n",rom_name[i]);
			return 1;
		 }

		 size = osd_fsize(fp);

		 if ((cp=strrchr(rom_name[i],'.'))!=NULL) {
			if ((cp[1]!=0)&&(cp[2]=='0')&&(cp[3]==0)) {
				switch(toupper(cp[1])) {
				case 'A': addr=0xa000;break;
				case '2': addr=0x2000;break;
				case '4': addr=0x4000;break;
				case '6': addr=0x6000;break;
				}
			} else {
				if (stricmp(cp,".prg")==0) {
					unsigned short in;
					osd_fread_lsbfirst(fp,&in, 2);
					if (errorlog) fprintf(errorlog,"rom prg %.4x\n",in);
					addr=in;
					size-=2;
				}
			}
		 }
		 if (addr==0) {
			 if (size==0x4000) { // I think rom at 0x4000
				 addr=0x4000;
			 } else {
				addr=0xa000;
			 }
		 }

		 if (errorlog)
			fprintf(errorlog,"loading rom %s at %.4x size:%.4x\n",
						rom_name[i],addr, size);
		 read=osd_fread(fp,vc20_memory+addr,size);
		 osd_fclose(fp);
		 if (read!=size) return 1;
	 }
	 return result;
}

int 	vc20_rom_id(const char *name, const char *gamename)
{
	FILE *romfile;
	unsigned char magic[]={ 0x41,0x30,0x20,0xc3,0xc2,0xcd }; // A0 CBM at 0xa004 (module offset 4)
	unsigned char buffer[sizeof(magic)];
	char *cp;
	int retval;

	if (errorlog) fprintf(errorlog,"vc20_rom_id %s\n",gamename);
	if (!(romfile = osd_fopen (name, name, OSD_FILETYPE_IMAGE_R, 0))) {
		if (errorlog) fprintf(errorlog,"rom %s not found\n",name);
		return 0;
	}

	retval = 0;

	osd_fseek (romfile, 4, SEEK_SET);
	osd_fread (romfile, buffer, sizeof(magic));
	osd_fclose (romfile);

	if (!memcmp(buffer,magic,sizeof(magic))) retval=1;

	if ((cp=strrchr(rom_name[0],'.'))!=NULL) {
		if ((stricmp(cp+1,"a0")==0)
			||(stricmp(cp+1,"20")==0)
			||(stricmp(cp+1,"40")==0)
			||(stricmp(cp+1,"60")==0)
			||(stricmp(cp+1,"bin")==0)
			||(stricmp(cp+1,"rom")==0)
			||(stricmp(cp+1,"prg")==0) ) retval=1;
	}

	if (errorlog) {
		if (retval) fprintf(errorlog,"rom %s recognized\n",name);
		else fprintf(errorlog,"rom %s not recognized\n",name);
	}

	return retval;
}

int vc20_frame_interrupt (void)
{
	int value;

	via_0_ca1_w(0,via0_read_ca1(0) );

	value=0xff;
	if (KEY_DEL) value&=~0x80;
	if (KEY_POUND) value&=~0x40;
	if (KEY_PLUS) value&=~0x20;
	if (KEY_9) value&=~0x10;
	if (KEY_7) value&=~8;
	if (KEY_5) value&=~4;
	if (KEY_3) value&=~2;
	if (KEY_1) value&=~1;
	keyline[0]=value;

	value=0xff;
	if (KEY_RETURN) value&=~0x80;
	if (KEY_ASTERIX) value&=~0x40;
	if (KEY_P) value&=~0x20;
	if (KEY_I) value&=~0x10;
	if (KEY_Y) value&=~8;
	if (KEY_R) value&=~4;
	if (KEY_W) value&=~2;
	if (KEY_ARROW_LEFT) value&=~1;
	keyline[1]=value;

	value=0xff;
	if (KEY_CURSOR_RIGHT) value&=~0x80;
	if (KEY_SEMICOLON) value&=~0x40;
	if (KEY_L) value&=~0x20;
	if (KEY_J) value&=~0x10;
	if (KEY_G) value&=~8;
	if (KEY_D) value&=~4;
	if (KEY_A) value&=~2;
	if (KEY_CTRL) value&=~1;
	keyline[2]=value;

	value=0xff;
	if (KEY_CURSOR_DOWN) value&=~0x80;
	if (KEY_SLASH) value&=~0x40;
	if (KEY_COMMA) value&=~0x20;
	if (KEY_N) value&=~0x10;
	if (KEY_V) value&=~8;
	if (KEY_X) value&=~4;
	if (KEY_LEFT_SHIFT) value&=~2;
	if (KEY_STOP) value&=~1;
	keyline[3]=value;

	value=0xff;
	if (KEY_F1) value&=~0x80;
	if (KEY_RIGHT_SHIFT) value&=~0x40;
	if (KEY_POINT) value&=~0x20;
	if (KEY_M) value&=~0x10;
	if (KEY_B) value&=~8;
	if (KEY_C) value&=~4;
	if (KEY_Z) value&=~2;
	if (KEY_SPACE) value&=~1;
	keyline[4]=value;

	value=0xff;
	if (KEY_F3) value&=~0x80;
	if (KEY_EQUALS) value&=~0x40;
	if (KEY_COLON) value&=~0x20;
	if (KEY_K) value&=~0x10;
	if (KEY_H) value&=~8;
	if (KEY_F) value&=~4;
	if (KEY_S) value&=~2;
	if (KEY_CBM) value&=~1;
	keyline[5]=value;

	value=0xff;
	if (KEY_F5) value&=~0x80;
	if (KEY_ARROW_UP) value&=~0x40;
	if (KEY_ATSIGN) value&=~0x20;
	if (KEY_O) value&=~0x10;
	if (KEY_U) value&=~8;
	if (KEY_T) value&=~4;
	if (KEY_E) value&=~2;
	if (KEY_Q) value&=~1;
	keyline[6]=value;

	value=0xff;
	if (KEY_F7) value&=~0x80;
	if (KEY_HOME) value&=~0x40;
	if (KEY_MINUS) value&=~0x20;
	if (KEY_0) value&=~0x10;
	if (KEY_8) value&=~8;
	if (KEY_6) value&=~4;
	if (KEY_4) value&=~2;
	if (KEY_2) value&=~1;
	keyline[7]=value;

	return ignore_interrupt ();
}
