#ifndef __VC20_H_
#define __VC20_H_

#include <stdio.h>
#include "osd_cpu.h"
#include "driver.h"
#include "vidhrdw/generic.h"
#include "mess/vidhrdw/vic6560.h"

/* enable and set level for verbosity of the various parts of emulation */


#define VERBOSE_DBG 0       /* general debug messages */

/**************************************************************************
 * Logging
 * call the XXX_LOG with XXX_LOG("info",(errorlog,"%fmt\n",args));
 * where "info" can also be 0 to append .."%fmt",args to a line.
 **************************************************************************/
#define LOG(LEVEL,N,M,A)  \
	if( errorlog && (LEVEL>=N) ){ if( M )fprintf( errorlog,"%11.6f: %-24s",timer_get_time(),(char*)M ); fprintf##A; }

#if VERBOSE_DBG
#define DBG_LOG(n,m,a) LOG(VERBOSE_DBG,n,m,a)
#else
#define DBG_LOG(n,m,a)
#endif

#if VERBOSE_DBG
#define VIC_LOG(n,m,a) LOG(VERBOSE_DBG,n,m,a)
#else
#define VIC_LOG(n,m,a)
#endif

#if VERBOSE_DBG
#define VIA_LOG(n,m,a) LOG(VERBOSE_DBG,n,m,a)
#else
#define VIA_LOG(n,m,a)
#endif

#if VERBOSE_DBG
#define SND_LOG(n,m,a) LOG(VERBOSE_DBG,n,m,a)
#else
#define SND_LOG(n,m,a)
#endif

// macros to access keys
#define JOYSTICK_LEFT	(input_port_0_r(0)&0x80)
#define JOYSTICK_RIGHT	((input_port_0_r(0)&0x40))
#define JOYSTICK_UP		((input_port_0_r(0)&0x20))
#define JOYSTICK_DOWN	((input_port_0_r(0)&0x10))
#define JOYSTICK_BUTTON ((input_port_0_r(0)&8))
#define PADDLE1_BUTTON	((input_port_1_r(0)&0x100))
#define PADDLE1_VALUE	(input_port_1_r(0)&0xff)
#define PADDLE2_BUTTON	((input_port_2_r(0)&0x100))
#define PADDLE2_VALUE	(input_port_2_r(0)&0xff)

#define KEY_ARROW_LEFT (input_port_3_r(0)&0x8000)
#define KEY_1 (input_port_3_r(0)&0x4000)
#define KEY_2 (input_port_3_r(0)&0x2000)
#define KEY_3 (input_port_3_r(0)&0x1000)
#define KEY_4 (input_port_3_r(0)&0x800)
#define KEY_5 (input_port_3_r(0)&0x400)
#define KEY_6 (input_port_3_r(0)&0x200)
#define KEY_7 (input_port_3_r(0)&0x100)
#define KEY_8 (input_port_3_r(0)&0x80)
#define KEY_9 (input_port_3_r(0)&0x40)
#define KEY_0 (input_port_3_r(0)&0x20)
#define KEY_PLUS (input_port_3_r(0)&0x10)
#define KEY_MINUS (input_port_3_r(0)&8)
#define KEY_POUND (input_port_3_r(0)&4)
#define KEY_HOME (input_port_3_r(0)&2)
#define KEY_DEL (input_port_3_r(0)&1)

#define KEY_CTRL (input_port_4_r(0)&0x8000)
#define KEY_Q (input_port_4_r(0)&0x4000)
#define KEY_W (input_port_4_r(0)&0x2000)
#define KEY_E (input_port_4_r(0)&0x1000)
#define KEY_R (input_port_4_r(0)&0x800)
#define KEY_T (input_port_4_r(0)&0x400)
#define KEY_Y (input_port_4_r(0)&0x200)
#define KEY_U (input_port_4_r(0)&0x100)
#define KEY_I (input_port_4_r(0)&0x80)
#define KEY_O (input_port_4_r(0)&0x40)
#define KEY_P (input_port_4_r(0)&0x20)
#define KEY_ATSIGN (input_port_4_r(0)&0x10)
#define KEY_ASTERIX (input_port_4_r(0)&8)
#define KEY_ARROW_UP (input_port_4_r(0)&4)
#define KEY_RESTORE (input_port_4_r(0)&2)
#define KEY_STOP (input_port_4_r(0)&1)

#define KEY_SHIFTLOCK (input_port_5_r(0)&0x8000)
#define KEY_A (input_port_5_r(0)&0x4000)
#define KEY_S (input_port_5_r(0)&0x2000)
#define KEY_D (input_port_5_r(0)&0x1000)
#define KEY_F (input_port_5_r(0)&0x800)
#define KEY_G (input_port_5_r(0)&0x400)
#define KEY_H (input_port_5_r(0)&0x200)
#define KEY_J (input_port_5_r(0)&0x100)
#define KEY_K (input_port_5_r(0)&0x80)
#define KEY_L (input_port_5_r(0)&0x40)
#define KEY_SEMICOLON (input_port_5_r(0)&0x20)
#define KEY_COLON (input_port_5_r(0)&0x10)
#define KEY_EQUALS (input_port_5_r(0)&8)
#define KEY_RETURN (input_port_5_r(0)&4)
#define KEY_CBM (input_port_5_r(0)&2)
#define KEY_LEFT_SHIFT ((input_port_5_r(0)&1)||KEY_SHIFTLOCK)

#define KEY_F5 (input_port_7_r(0)&0x8000)
#define KEY_F7 (input_port_7_r(0)&0x4000)
#define KEY_CURSOR_UP (input_port_7_r(0)&0x2000)
#define KEY_CURSOR_LEFT (input_port_7_r(0)&0x1000)

#define KEY_Z (input_port_6_r(0)&0x8000)
#define KEY_X (input_port_6_r(0)&0x4000)
#define KEY_C (input_port_6_r(0)&0x2000)
#define KEY_V (input_port_6_r(0)&0x1000)
#define KEY_B (input_port_6_r(0)&0x800)
#define KEY_N (input_port_6_r(0)&0x400)
#define KEY_M (input_port_6_r(0)&0x200)
#define KEY_COMMA (input_port_6_r(0)&0x100)
#define KEY_POINT (input_port_6_r(0)&0x80)
#define KEY_SLASH (input_port_6_r(0)&0x40)
#define KEY_RIGHT_SHIFT ((input_port_6_r(0)&0x20)\
								||KEY_CURSOR_UP||KEY_CURSOR_LEFT)
#define KEY_CURSOR_DOWN ((input_port_6_r(0)&0x10)||KEY_CURSOR_UP)
#define KEY_CURSOR_RIGHT ((input_port_6_r(0)&8)||KEY_CURSOR_LEFT)
#define KEY_SPACE (input_port_6_r(0)&4)
#define KEY_F1 (input_port_6_r(0)&2)
#define KEY_F3 (input_port_6_r(0)&1)


// macros to access the dip switches
#define EXPANSION (input_port_8_r(0)&7)
#define EXP_3K 1
#define EXP_8K 2
#define EXP_16K 3
#define EXP_32K 4
#define EXP_CUSTOM 5
#define RAMIN0X0400 ((EXPANSION==EXP_3K)\
							||((EXPANSION==EXP_CUSTOM)&&(input_port_8_r(0)&8)) )
#define RAMIN0X2000 ((EXPANSION==EXP_8K)||(EXPANSION==EXP_16K)\
							||(EXPANSION==EXP_32K)\
							||((EXPANSION==EXP_CUSTOM)&&(input_port_8_r(0)&0x10)) )
#define RAMIN0X4000 ((EXPANSION==EXP_16K)||(EXPANSION==EXP_32K)\
							||((EXPANSION==EXP_CUSTOM)&&(input_port_8_r(0)&0x20)) )
#define RAMIN0X6000 ((EXPANSION==EXP_32K)\
							||((EXPANSION==EXP_CUSTOM)&&(input_port_8_r(0)&0x40)) )
#define RAMIN0XA000 ((EXPANSION==EXP_32K)\
							||((EXPANSION==EXP_CUSTOM)&&(input_port_8_r(0)&0x80)) )

#define JOYSTICK (input_port_8_r(0)&0x100)
#define PADDLES (input_port_8_r(0)&0x200)
#define LIGHTPEN (input_port_8_r(0)&0x400)

#define JOY_BUTTON ( ( JOYSTICK&&(input_port_0_r(0)&8) )\
							||(LIGHTPEN&&0) )
#define JOY_LEFT ( ( JOYSTICK&&(input_port_0_r(0)&0x80) )\
						 ||( PADDLES&&(input_port_1_r(0)&0x100) ) )
#define JOY_RIGHT ( ( JOYSTICK&&(input_port_0_r(0)&0x40) )\
						||( PADDLES&&(input_port_2_r(0)&0x100) ) )
#define JOY_UP (JOYSTICK&&(input_port_0_r(0)&0x20))
#define JOY_DOWN (JOYSTICK&&(input_port_0_r(0)&0x10))

#define VC20ADDR2VIC6560ADDR(a) (((a)>0x8000)?((a)&0x1fff):((a)|0x2000))
#define VIC6560ADDR2VC20ADDR(a) (((a)>0x2000)?((a)&0x1fff):((a)|0x8000))

extern UINT8 *vc20_memory;
extern UINT8 *vc20_memory_1000;
extern UINT8 *vc20_memory_9400;

extern void vc20_write(int offset, int data);
extern void vc20_write_0400(int offset, int data);
extern void vc20_write_1000(int offset, int data);
extern void vc20_write_9400(int offset, int data);

// VIC reads 12 bit
extern int vic6560_dma_read(int offset);

extern int  vc20_rom_load(void);
extern int  vc20_rom_id(const char *name, const char *gamename);

extern void vc20_init_machine(void);
extern void vic20_init_machine(void);
extern void vc20_shutdown_machine(void);

extern int  vc20_frame_interrupt(void);

#endif
