/* Oric 1 & Oric Atmos Machine Drivers ..... */

#include "driver.h"
#include "vidhrdw/generic.h"

/* from machine/oric.c */

void oric_init_machine (void);
void oric_shutdown_machine (void);
int oric_load_rom (void);
int oric_interrupt(void);
int oric_id_rom (const char *name, const char *gamename);
int oric_IO_r (int offset);
void oric_IO_w (int offset, int data);



/* from vidhrdw/oric.c */
void oric_vh_screenrefresh(struct osd_bitmap *bitmap, int full_refresh);
int oric_vh_start(void);
void oric_vh_stop(void);

unsigned char *oric_IO;

// int oric_ram_r (int offset);
// void oric_ram_w (int offset, int data);

static struct MemoryReadAddress oric_readmem[] =
{
    /* { 0x0000, 0x04ff, oric_io_r, &oric_ram }, */
    /* { 0x0000, 0xBFFF, oric_ram_r, &oric_ram }, */
    { 0x0000, 0x02FF, MRA_RAM }, 
    { 0x0300, 0x03ff, oric_IO_r },
    //{ 0x0300, 0x03ff, MRA_RAM },
    { 0x0400, 0xBFFF, MRA_RAM }, 
    { 0xc000, 0xFFFF, MRA_ROM },
    { -1 }
};

static struct MemoryWriteAddress oric_writemem[] =
{
    /* { 0x0000, 0x04ff, oric_IO_w, &oric_ram }, */
    /* { 0x0000, 0xbFFF, oric_ram_w, &oric_ram }, */
    { 0x0000, 0x02FF, MWA_RAM },
    { 0x0300, 0x03ff, oric_IO_w },
    //{ 0x0300, 0x03ff, MWA_RAM },
    { 0x0400, 0xbFFF, MWA_RAM },
    { 0xc000, 0xFFFF, MWA_ROM },
    { -1 }
};

static const char *oric_file_extensions[] = { "tap",0 };

static unsigned char oric_palette[16*3] = {
        0x00, 0x00, 0x00, 0xcf, 0x00, 0x00,
        0x00, 0xcf, 0x00, 0xcf, 0xcf, 0x00,
        0x00, 0x00, 0xcf, 0xcf, 0x00, 0xcf,
        0x00, 0xcf, 0xcf, 0xcf, 0xcf, 0xcf,

        0x00, 0x00, 0x00, 0xff, 0x00, 0x00,
        0x00, 0xff, 0x00, 0xff, 0xff, 0x00,
        0x00, 0x00, 0xff, 0xff, 0x00, 0xff,
        0x00, 0xff, 0xff, 0xff, 0xff, 0xff,
};

static unsigned short oric_colortable[128*2] = {
	0,0 , 0,1, 0,2, 0,3, 0,4, 0,5, 0,6, 0,7,
        1,0 , 2,1, 2,2, 1,3, 1,4, 1,5, 1,6, 1,7,
        2,0 , 4,1, 4,2, 1,3, 1,4, 1,5, 1,6, 1,7,
        3,0 , 6,1, 6,2, 1,3, 1,4, 1,5, 1,6, 1,7,
        4,0 , 1,1, 1,2, 1,3, 1,4, 1,5, 1,6, 1,7,
        5,0 , 3,1, 3,2, 1,3, 1,4, 1,5, 1,6, 1,7,
        6,0 , 6,1, 6,2, 1,3, 1,4, 1,5, 1,6, 1,7,
        7,0 , 7,1, 7,2, 1,3, 1,4, 1,5, 1,6, 1,7,

	8,8  , 8,9,  8,10,  8 ,11, 8,12,  8,13, 8,14, 8,15,
	9,8  , 9,9,  9,10,  9 ,11, 9,12,  9,13, 9,14, 9,15,
	10,8 , 10,9, 10,10, 10,11, 10,12, 10,13, 10,14, 10,15,
	11,8 , 11,9, 11,10, 11,11, 11,12, 11,13, 11,14, 11,15,
	12,8 , 12,9, 12,10, 12,11, 12,12, 12,13, 12,14, 12,15,
	13,8 , 13,9, 13,10, 13,11, 13,12, 13,13, 13,14, 13,15,
	14,8 , 14,9, 14,10, 14,11, 14,12, 14,13, 14,14, 14,15,
	15,8 , 15,9, 15,10, 15,11, 15,12, 15,13, 15,14, 15,15
};

/* Initialise the palette */
static void oric_init_palette(unsigned char *sys_palette, unsigned short *sys_colortable,const unsigned char *color_prom)
{
        memcpy(sys_palette,oric_palette,sizeof(oric_palette));
        memcpy(sys_colortable,oric_colortable,sizeof(oric_colortable));
}

static struct GfxDecodeInfo oric_gfxdecodeinfo[] = {
	{ -1 } /* end of array */
};

/* read PSG port A */
int     oric_psg_porta_read(int offset)
{
        return 0;
}


static struct AY8910interface oric_ay_interface =
{
	1,	/* 1 chips */
	1000000,	/* 1.0 MHz  */
        { 25,25},
	AY8910_DEFAULT_GAIN,/* ???????? */
        { oric_psg_porta_read },
	{ 0 },
	{ 0 },
	{ 0 }
};


static struct MachineDriver oric_machine_driver =
{
	/* basic machine hardware */
	{
		{
                        CPU_M6502,
            1000000,
			0,
                        oric_readmem,oric_writemem,0,0,
                        oric_interrupt, 10,
			0, 0
		}
	},
	60, DEFAULT_REAL_60HZ_VBLANK_DURATION,	/* frames per second, vblank duration */
	1,
        oric_init_machine, /* init_machine */
        oric_shutdown_machine, /* stop_machine */

	/* video hardware */
        40*6,                                /* screen width */
        28*8,                                /* screen height */
        { 0, 40*6-1, 0, 28*8-1},             /* visible_area */
        oric_gfxdecodeinfo,                  /* graphics decode info */
        256, 128,                             /* colors used for the characters */
        oric_init_palette,                                   /* convert color prom */

	VIDEO_TYPE_RASTER,




        // VIDEO_TYPE_RASTER | VIDEO_MODIFIES_PALETTE,
	0,
        oric_vh_start,
        oric_vh_stop, 
        oric_vh_screenrefresh,

	/* sound hardware */
	0,0,0,0,
	{
		{
                        SOUND_AY8910,
                        &oric_ay_interface
		}
	}
};


ROM_START(oric1)
        ROM_REGION (0x10000)
        ROM_LOAD ("oric1.rom", 0xc000, 0x4000, 0xf18710b4)
ROM_END

ROM_START(orica)
        ROM_REGION (0x10000)
        ROM_LOAD ("orica.rom", 0xc000, 0x4000, 0xc3a92bef)
ROM_END

struct GameDriver oric1_driver =
{
	__FILE__,
	0,
        "oric1",
        "Oric 1",
        "1983",
        "Oric",
    "Paul Cook",
    0,
        &oric_machine_driver,
	0,

        rom_oric1,
        0, // oric_load_rom,
        0, // oric_id_rom,
        oric_file_extensions,
        0,      /* number of ROM slots */
        0,      /* number of floppy drives supported */
	0,	/* number of hard drives supported */
        0,      /* number of cassette drives supported */
	0, 0,
	0,
	0,	/* sound_prom */

        0, // input_ports,

        0,
        // changed for port to 36b10 source ...
        0, // oric_palette,
        0, // oric_colortable,

        GAME_COMPUTER | ORIENTATION_DEFAULT,

	0, 0,
};

struct GameDriver orica_driver =
{
	__FILE__,
	0,
        "orica",
        "Oric Atmos",
        "1983",
        "Oric",
    "Paul Cook",
    0,
        &oric_machine_driver,
	0,

        rom_orica,
        0, // oric_load_rom,
        0, // oric_id_rom,
        oric_file_extensions,
        0,      /* number of ROM slots */
        0,      /* number of floppy drives supported */
	0,	/* number of hard drives supported */
        0,      /* number of cassette drives supported */
	0, 0,
	0,
	0,	/* sound_prom */

        0, // input_ports,

        0,
        // changed for port to 36b10 source ...
        0, // oric_palette,
        0, // oric_colortable,
        
        GAME_COMPUTER | ORIENTATION_DEFAULT,

	0, 0,
};







